@@ -5,88 +5,95 @@
  * Fontmap implementation
  *
  * Authors:
- *   Lauris Kaplinski <lauris@helixcode.com>
+ *   Lauris Kaplinski <lauris@ximian.com>
  *
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000-2001 Ximian, Inc.
+ *
+ * TODO: Recycle font entries, if they are identical for different maps
  *
  */
 
-#include <glib/gmacros.h>
+#include <glib.h>
 
-G_END_DECLS
+G_BEGIN_DECLS
 
 typedef struct _GPFontMap GPFontMap;
 typedef struct _GPFileEntry GPFileEntry;
 typedef struct _GPFontEntry GPFontEntry;
 typedef struct _GPFamilyEntry GPFamilyEntry;
 typedef struct _GPFontEntryT1 GPFontEntryT1;
-#if 0
 typedef struct _GPFontEntryT1Alias GPFontEntryT1Alias;
-#endif
 typedef struct _GPFontEntryTT GPFontEntryTT;
 typedef struct _GPFontEntryAlias GPFontEntryAlias;
+typedef struct _GPFontEntrySpecial GPFontEntrySpecial;
 
 #include <sys/types.h>
-#include <glib.h>
 #include <libxml/tree.h>
 #include "gnome-font-face.h"
-#include "gnome-font-compat.h"
 
 typedef enum {
 	GP_FONT_ENTRY_UNKNOWN,
 	GP_FONT_ENTRY_TYPE1, /* Just ordinary Type1 font */
-#if 0
+	GP_FONT_ENTRY_TRUETYPE,
 	GP_FONT_ENTRY_TYPE1_ALIAS, /* Type1 font with foreign afm */
-#endif
-	GP_FONT_ENTRY_TRUETYPE, /* TrueType font (with 'glyf' table, no collections) */
-	GP_FONT_ENTRY_ALIAS /* Full alias */
+	GP_FONT_ENTRY_ALIAS, /* Full alias */
+	GP_FONT_ENTRY_SPECIAL
 } GPFontEntryType;
 
 struct _GPFileEntry {
 	gchar * name;
-	size_t size;
-	time_t mtime;
 };
 
 struct _GPFontMap {
 	gint refcount;
 	gint num_fonts;
-	GPFileEntry user;
-	GPFileEntry system;
+	/* Timestamp of STATIC dir */
+	time_t mtime_static;
+	/* Timestamp of DYNAMIC dir */
+	time_t mtime_dynamic;
+	/* Timestamp of USER dir */
+	time_t mtime_user;
+
 	/* Name -> FontEntry */
-	GHashTable * fontdict;
+	GHashTable *fontdict;
 	/* Family name -> FamilyEntry */
-	GHashTable * familydict;
+	GHashTable *familydict;
 	/* List of FontEntries, sorted A-Z */
-	GSList * fonts;
+	GSList *fonts;
 	/* List of FamilyEntries, sorted A-Z */
-	GSList * families;
+	GSList *families;
 	/* List of font names (pointing to entry->name) */
-	GList * fontlist;
+	GList *fontlist;
 	/* List of family names (pointing to entry->name */
-	GList * familylist;
+	GList *familylist;
+
+	/* List of default entries (only for parsing, will be cleaned later) */
+	GSList *defaults;
+	/* Dictionary of defaults by language_COUNTRY code */
+	GHashTable *defaultsdict;
 };
 
 struct _GPFontEntry {
 	GPFontEntryType type;
 	gint refcount;
-	guint hidden : 1;
 	/* Our face */
 	GnomeFontFace * face;
 	/* Common fields */
-	gchar * name;
-	gchar * version;
-	gchar * familyname;
-	gchar * speciesname;
-	gchar * psname;
+	gchar *name;
+	gchar *version;
+	gchar *familyname;
+	gchar *speciesname;
+	gchar *psname;
 	/* fixme: fixme: fixme: */
-	gchar * weight;
+	gchar *weight;
+	/* Some parsed afm latin metrics */
+	GnomeFontWeight Weight;
+	gdouble ItalicAngle; /* italic < 0 */
 };
 
 struct _GPFamilyEntry {
 	gint refcount;
 	gchar * name;
-	guint hidden : 1;
 	/* List of FontEntries */
 	GSList * fonts;
 };
@@ -95,24 +102,17 @@ struct _GPFontEntryT1 {
 	GPFontEntry entry;
 	GPFileEntry afm;
 	GPFileEntry pfb;
-	/* Some parsed afm latin metrics */
-	GnomeFontWeight Weight;
-	gdouble ItalicAngle; /* italic < 0 */
 };
 
-#if 0
 struct _GPFontEntryT1Alias {
 	GPFontEntryT1 t1;
 	gchar * alias;
 };
-#endif
 
 struct _GPFontEntryTT {
 	GPFontEntry entry;
 	GPFileEntry ttf;
-	/* fixme: */
-	GnomeFontWeight Weight;
-	gdouble ItalicAngle; /* italic < 0 */
+	gint facenum;
 };
 
 struct _GPFontEntryAlias {
@@ -120,12 +120,25 @@ struct _GPFontEntryAlias {
 	GPFontEntry * ref;
 };
 
+struct _GPFontEntrySpecial {
+	GPFontEntry entry;
+	GPFileEntry file;
+	gint subface;
+	GSList *additional;
+};
+
 GPFontMap * gp_fontmap_get (void);
 void gp_fontmap_release (GPFontMap *);
 
 void gp_font_entry_ref (GPFontEntry * entry);
 void gp_font_entry_unref (GPFontEntry * entry);
 
+/* This is experimental method (not public anyways) (Lauris) */
+
+GPFontEntry *gp_font_entry_from_files (GPFontMap *map,
+				       const guchar *name, const guchar *family, const guchar *species, gboolean hidden,
+				       const guchar *filename, gint face, const GSList *additional);
+
 GnomeFontWeight gp_fontmap_lookup_weight (const gchar * weight);
 
 G_END_DECLS
