@@ -1,14 +1,28 @@
 #define __GPA_ROOT_C__
 
 /*
- * GPARoot
+ * This file is part of libgnomeprint 2
  *
- * Opaque root object to gnome-print configuration tree
+ * Libgnomeprint is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- * Authors:
+ * Libgnomeprint is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with the libgnomeprint; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Authors :
+ *   Jose M. Celorio <chema@ximian.com>
  *   Lauris Kaplinski <lauris@ximian.com>
  *
- * Copyright (C) 2001 Ximian, Inc.
+ * Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
  *
  */
 
@@ -28,7 +42,7 @@ static void gpa_root_finalize (GObject *object);
 
 static GPANode *gpa_root_get_child (GPANode *node, GPANode *ref);
 static GPANode *gpa_root_lookup (GPANode *node, const guchar *path);
-static void gpa_root_modified (GPANode *node);
+static void gpa_root_modified (GPANode *node, guint flags);
 
 static GPANode *root_instance = NULL;
 
@@ -152,17 +166,17 @@ gpa_root_lookup (GPANode *node, const guchar *path)
 }
 
 static void
-gpa_root_modified (GPANode *node)
+gpa_root_modified (GPANode *node, guint flags)
 {
 	GPARoot *root;
 
 	root = GPA_ROOT (node);
 
 	if (root->vendors && (GPA_NODE_FLAGS (root->vendors) & GPA_MODIFIED_FLAG)) {
-		gpa_node_emit_modified (GPA_NODE (root->vendors));
+		gpa_node_emit_modified (GPA_NODE (root->vendors), 0);
 	}
 	if (root->printers && (GPA_NODE_FLAGS (root->printers) & GPA_MODIFIED_FLAG)) {
-		gpa_node_emit_modified (GPA_NODE (root->printers));
+		gpa_node_emit_modified (GPA_NODE (root->printers), 0);
 	}
 }
 
