@@ -1,26 +1,32 @@
 #define __GPA_UTILS_C__
 
-/*                                                            
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+/*
+ * This file is part of libgnomeprint 2
  *
- * This program is distributed in the hope that it will be useful,
+ * Libgnomeprint is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Libgnomeprint is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with the libgnomeprint; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors :
- *   Chema Celorio <chema@celorio.com>
+ *   Jose M. Celorio <chema@ximian.com>
  *   Lauris Kaplinski <lauris@ximian.com>
  *
  * Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
  *
+ * Contains pieces of code from glib, Copyright GLib Team and
+ * others 1997-2000.
+ *
  */
 
 #include <string.h>
@@ -30,6 +36,8 @@
 #include "gpa-node-private.h"
 #include "gpa-utils.h"
 
+#define noGPA_QUARK_DEBUG
+
 guchar *
 gpa_id_new (const guchar *key)
 {
@@ -208,6 +216,85 @@ gpa_xml_node_get_child (xmlNodePtr node, const guchar *name)
 }
 
 
+/* Private quarks */
+
+#define GPA_QUARK_BLOCK_SIZE 128
+static GHashTable *qdict = NULL;
+static const guchar **qarray = NULL;
+static GQuark qseq = 0;
+
+static GPAQuark
+gpa_quark_new (const guchar *string)
+{
+	if ((qseq % GPA_QUARK_BLOCK_SIZE) == 0) {
+		qarray = g_renew (const guchar *, qarray, qseq + GPA_QUARK_BLOCK_SIZE);
+#ifdef GPA_QUARK_DEBUG
+		g_print ("GPAQuark: Increasing buffer, now %d\n", qseq + GPA_QUARK_BLOCK_SIZE);
+#endif
+	}
+
+	qarray[qseq++] = string;
+
+	g_hash_table_insert (qdict, (gpointer) string, GUINT_TO_POINTER (qseq));
+
+	g_print ("GPAQuark: New quark %d:%s\n", qseq, string);
+
+	return qseq;
+}
+
+GPAQuark
+gpa_quark_try_string (const guchar *string)
+{
+	GPAQuark q;
+
+	g_return_val_if_fail (string != NULL, 0);
+
+	q = (qdict) ? GPOINTER_TO_INT (g_hash_table_lookup (qdict, string)) : 0;
+
+	return q;
+}
+
+GPAQuark
+gpa_quark_from_string (const guchar *string)
+{
+	GPAQuark q;
+
+	g_return_val_if_fail (string != NULL, 0);
+
+	if (!qdict) qdict = g_hash_table_new (g_str_hash, g_str_equal);
+
+	q = GPOINTER_TO_INT (g_hash_table_lookup (qdict, string));
+
+	if (!q) q = gpa_quark_new (g_strdup (string));
+
+	return q;
+}
+
+GPAQuark
+gpa_quark_from_static_string (const guchar *string)
+{
+	GPAQuark q;
+
+	g_return_val_if_fail (string != NULL, 0);
+
+	if (!qdict) qdict = g_hash_table_new (g_str_hash, g_str_equal);
+
+	q = GPOINTER_TO_INT (g_hash_table_lookup (qdict, string));
+
+	if (!q) q = gpa_quark_new (string);
+
+	return q;
+}
+
+const guchar *
+gpa_quark_to_string (GPAQuark quark)
+{
+	if ((quark > 0) && (quark <= qseq)) return qarray[quark - 1];
+
+	return NULL;
+}
+
+
 #if 0
 static void
 gpa_hash_item_copy (gpointer key_in, gpointer value_in, gpointer data)
